require "import"

import "android.content.Context"
import "android.content.ClipData"
import "android.content.ClipboardManager"
import "android.view.accessibility.AccessibilityNodeInfo"

-- Retrieve the context from the service
local context = service.getBaseContext()

-- Get the ClipboardManager service
local clipboard = context.getSystemService(Context.CLIPBOARD_SERVICE)

-- Get the root of the accessibility node tree
local rootNode = service.getRootInActiveWindow()

if rootNode then
    -- Find the currently focused node
    local focusedNode = rootNode.findFocus(AccessibilityNodeInfo.FOCUS_ACCESSIBILITY)

    if focusedNode then
        -- Recursively extract all text from the focused node and its children
        local function extractText(node)
            local text = ""

            -- Check if node.getText() is not nil and convert to string
            local nodeText = node.getText()
            if nodeText then
                text = text .. tostring(nodeText) .. "\n"
            end

            -- Iterate through child nodes
            for i = 0, node.getChildCount() - 1 do
                local child = node.getChild(i)
                if child then
                    text = text .. extractText(child)
                end
            end

            return text
        end

        local textToCopy = extractText(focusedNode)

        if textToCopy and textToCopy ~= "" then
            -- Copy the text to the clipboard
            local clip = ClipData.newPlainText("label", textToCopy)
            clipboard.setPrimaryClip(clip)

            -- Notify the user that the text has been copied
service.speak("text copied.")        else
            -- Handle cases where the focused element has no text
            service.speak("The focused element has no text to copy.")
        end
    else
        -- Handle cases where no element is focused
        service.speak("No element is currently focused.")
    end
else
    -- Handle cases where the root node is not accessible
    service.speak("Unable to access the root node. Ensure the accessibility service is active.")
end